Class: Analyzer

Inherits:
Object
  • Object
show all
Defined in:
lib/production_log/analyzer.rb

Overview

Calculates statistics for production logs.

Constant Summary collapse

VERSION =

The version of the production log analyzer you are using.

'1.5.0'

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(logfile_name) ⇒ Analyzer

Creates a new Analyzer that will read data from logfile_name.



176
177
178
179
180
181
182
183
# File 'lib/production_log/analyzer.rb', line 176

def initialize(logfile_name)
  @logfile_name  = logfile_name
  @request_times = Hash.new { |h,k| h[k] = [] }
  @db_times      = Hash.new { |h,k| h[k] = [] }
  @row_counts    = Hash.new { |h,k| h[k] = [] }
  @query_counts  = Hash.new { |h,k| h[k] = [] }
  @render_times  = Hash.new { |h,k| h[k] = [] }
end

Instance Attribute Details

#db_timesObject (readonly)

An Array of all the request database times for the log file.



133
134
135
# File 'lib/production_log/analyzer.rb', line 133

def db_times
  @db_times
end

#logfile_nameObject (readonly)

The logfile being read by the Analyzer.



123
124
125
# File 'lib/production_log/analyzer.rb', line 123

def logfile_name
  @logfile_name
end

#query_countsObject (readonly)

Returns the value of attribute query_counts.



140
141
142
# File 'lib/production_log/analyzer.rb', line 140

def query_counts
  @query_counts
end

#render_timesObject (readonly)

An Array of all the request render times for the log file.



138
139
140
# File 'lib/production_log/analyzer.rb', line 138

def render_times
  @render_times
end

#request_timesObject (readonly)

An Array of all the request total times for the log file.



128
129
130
# File 'lib/production_log/analyzer.rb', line 128

def request_times
  @request_times
end

#row_countsObject (readonly)

Returns the value of attribute row_counts.



140
141
142
# File 'lib/production_log/analyzer.rb', line 140

def row_counts
  @row_counts
end

Class Method Details

.email(file_name, recipient, subject, count = 10) ⇒ Object

Generates and sends an email report with lots of fun stuff in it. This way, Mail.app will behave when given tabs.



146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
# File 'lib/production_log/analyzer.rb', line 146

def self.email(file_name, recipient, subject, count = 10)
  analyzer = self.new file_name
  analyzer.process
  body = analyzer.report count

  email = self.envelope(recipient, subject)
  email << nil
  email << "<pre>#{body}</pre>"
  email = email.join($/) << $/

  return email if $TESTING

  IO.popen("/usr/sbin/sendmail -i -t", "w+") do |sm|
    sm.print email
    sm.flush
  end
end

.envelope(recipient, subject = nil) ⇒ Object

:nodoc:



164
165
166
167
168
169
170
171
# File 'lib/production_log/analyzer.rb', line 164

def self.envelope(recipient, subject = nil) # :nodoc:
  envelope = {}
  envelope['To'] = recipient
  envelope['Subject'] = subject || "pl_analyze"
  envelope['Content-Type'] = "text/html"

  return envelope.map { |(k,v)| "#{k}: #{v}" }
end

Instance Method Details

#average_db_timeObject

The average total database time for all requests.



226
227
228
# File 'lib/production_log/analyzer.rb', line 226

def average_db_time
  return time_average(@db_times)
end

#average_render_timeObject

The average total render time for all requests.



247
248
249
# File 'lib/production_log/analyzer.rb', line 247

def average_render_time
  return time_average(@render_times)
end

#average_request_timeObject

The average total request time for all requests.



205
206
207
# File 'lib/production_log/analyzer.rb', line 205

def average_request_time
  return time_average(@request_times)
end

#db_time_std_devObject

The standard deviation of the total database time for all requests.



233
234
235
# File 'lib/production_log/analyzer.rb', line 233

def db_time_std_dev
  return time_std_dev(@db_times)
end

#db_times_summaryObject

A list of count/min/max/avg/std dev for database times.



275
276
277
# File 'lib/production_log/analyzer.rb', line 275

def db_times_summary
  return summarize("DB Times", @db_times)
end

#longest_request_nameObject

:nodoc:



384
385
386
387
388
389
390
391
392
393
394
395
396
# File 'lib/production_log/analyzer.rb', line 384

def longest_request_name # :nodoc:
  return @longest_req if defined? @longest_req

  names = @request_times.keys.map do |name|
    (name||'Unknown').length + 1 # + : - HACK where does nil come from?
  end

  @longest_req = names.max

  @longest_req = 'Unknown'.length + 1 if @longest_req.nil?

  return @longest_req
end

#pad_request_name(name) ⇒ Object

:nodoc:



398
399
400
401
402
403
# File 'lib/production_log/analyzer.rb', line 398

def pad_request_name(name) # :nodoc:
  name = (name||'Unknown') + ':' # HACK where does nil come from?
  padding_width = longest_request_name - name.length
  padding_width = 0 if padding_width < 0
  name += (' ' * padding_width)
end

#processObject

Processes the log file collecting statistics from each found LogEntry.



188
189
190
191
192
193
194
195
196
197
198
199
200
# File 'lib/production_log/analyzer.rb', line 188

def process
  File.open @logfile_name do |fp|
    LogParser.parse fp do |entry|
      entry_page = entry.page
      next if entry_page.nil?
      @request_times[entry_page] << entry.request_time
      @db_times[entry_page] << entry.db_time
      @row_counts[entry_page] << entry.row_count
      @query_counts[entry_page] << entry.query_count
      @render_times[entry_page] << entry.render_time
    end
  end
end

#render_time_std_devObject

The standard deviation of the total render time for all requests.



254
255
256
# File 'lib/production_log/analyzer.rb', line 254

def render_time_std_dev
  return time_std_dev(@render_times)
end

#render_times_summaryObject

A list of count/min/max/avg/std dev for request times.



282
283
284
# File 'lib/production_log/analyzer.rb', line 282

def render_times_summary
  return summarize("Render Times", @render_times)
end

#report(count) ⇒ Object

Builds a report containing count slow items.



289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
# File 'lib/production_log/analyzer.rb', line 289

def report(count)
  return "No requests to analyze" if request_times.empty?

  text = []

  text << request_times_summary
  text << nil
  text << "Slowest Request Times:"
  slowest_request_times(count).each do |time, name|
    text << "\t#{name} took #{'%0.3f' % time}s"
  end
  text << nil
  text << "-" * 72
  text << nil

  text << db_times_summary
  text << nil
  text << "Slowest Total DB Times:"
  slowest_db_times(count).each do |time, name|
    text << "\t#{name} took #{'%0.3f' % time}s"
  end
  text << nil
  text << "-" * 72
  text << nil

  text << render_times_summary
  text << nil
  text << "Slowest Total Render Times:"
  slowest_render_times(count).each do |time, name|
    text << "\t#{name} took #{'%0.3f' % time}s"
  end
  text << nil

  return text.join($/)
end

#request_time_std_devObject

The standard deviation of the total request time for all requests.



212
213
214
# File 'lib/production_log/analyzer.rb', line 212

def request_time_std_dev
  return time_std_dev(@request_times)
end

#request_times_summaryObject

A list of count/min/max/avg/std dev for request times.



268
269
270
# File 'lib/production_log/analyzer.rb', line 268

def request_times_summary
  return summarize("Request Times", @request_times)
end

#slowest_db_times(limit = 10) ⇒ Object

The limit slowest total database times.



240
241
242
# File 'lib/production_log/analyzer.rb', line 240

def slowest_db_times(limit = 10)
  return slowest_times(@db_times, limit)
end

#slowest_render_times(limit = 10) ⇒ Object

The limit slowest total render times for all requests.



261
262
263
# File 'lib/production_log/analyzer.rb', line 261

def slowest_render_times(limit = 10)
  return slowest_times(@render_times, limit)
end

#slowest_request_times(limit = 10) ⇒ Object

The limit slowest total request times.



219
220
221
# File 'lib/production_log/analyzer.rb', line 219

def slowest_request_times(limit = 10)
  return slowest_times(@request_times, limit)
end

#slowest_times(records, limit) ⇒ Object

:nodoc:



360
361
362
363
364
365
366
367
368
369
370
# File 'lib/production_log/analyzer.rb', line 360

def slowest_times(records, limit) # :nodoc:
  slowest_times = SlowestTimes.new limit

  records.each do |name, times|
    times.each do |time|
      slowest_times << [time, name]
    end
  end

  return slowest_times.sort_by { |time, name| time }.reverse
end

#summarize(title, records) ⇒ Object

:nodoc:



327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
354
355
356
357
358
# File 'lib/production_log/analyzer.rb', line 327

def summarize(title, records) # :nodoc:
  record = nil
  list = []

  # header
  record = [pad_request_name("#{title} Summary"), 'Count', 'Avg', 'Std Dev',
            'Min', 'Max', 'Queries', 'Rows']
  list << record.join("\t")

  # all requests
  times = records.values.flatten
  record = [times.average, times.standard_deviation, times.min, times.max, 0, 0]
  record.map! { |v| "%0.3f" % v }
  record.unshift [pad_request_name('ALL REQUESTS'), times.size]
  list << record.join("\t")

  # spacer
  list << nil

  records.sort_by { |k,v| v.size}.reverse_each do |req, times|
    if "DB Times" == title
      average_rows = @row_counts[req].average
      average_queries = @query_counts[req].average
    end
    record = [times.average, times.standard_deviation, times.min, times.max, average_queries || 0, average_rows || 0]
    record.map! { |v| "%0.3f" % v }
    record.unshift ["#{pad_request_name req}", times.size]
    list << record.join("\t")
  end

  return list.join("\n")
end

#time_average(records) ⇒ Object

:nodoc:



372
373
374
375
376
# File 'lib/production_log/analyzer.rb', line 372

def time_average(records) # :nodoc:
  times = records.values.flatten
  times.delete 0
  return times.average
end

#time_std_dev(records) ⇒ Object

:nodoc:



378
379
380
381
382
# File 'lib/production_log/analyzer.rb', line 378

def time_std_dev(records) # :nodoc:
  times = records.values.flatten
  times.delete 0
  return times.standard_deviation
end